//-----------------------------------------------------------------------------
#ifndef DeviceConfigureFrameH
#define DeviceConfigureFrameH DeviceConfigureFrameH
//-----------------------------------------------------------------------------
#include "wx/wx.h"
#include "DebugFileParser.h"
#include <mvIMPACT_CPP/mvIMPACT_acquire.h>
#include "ObjectFactory.h"
#include "DeviceHandler.h"
#ifdef BUILD_WITH_DIRECT_SHOW_SUPPORT
#	include "win32/DirectShowSupport.h"
#endif // #ifdef BUILD_WITH_DIRECT_SHOW_SUPPORT
class DeviceListCtrl;

//-----------------------------------------------------------------------------
class DeviceConfigureFrame : public wxFrame
//-----------------------------------------------------------------------------
{
public:
		DeviceConfigureFrame(const wxString& title, const wxPoint& pos, const wxSize& size);
	   ~DeviceConfigureFrame();
	typedef DeviceHandler* (*CreateDeviceHandler)(mvIMPACT::acquire::Device*);
	typedef ObjectFactory<DeviceHandler, wxString, CreateDeviceHandler, mvIMPACT::acquire::Device*> DeviceHandlerFactory;
	void ActivateDeviceInPropView( int deviceIndex );
	int SetID( int deviceIndex );
	int UpdateFirmware( int deviceIndex );
	int UpdateKernelDriver( int deviceIndex );
	void UpdateMenu( int deviceIndex );
	void WriteErrorMessage( const wxString& msg );
	void WriteLogMessage( const wxString& msg, const wxTextAttr& style = wxTextAttr(wxColour(0, 0, 0)) );
	DeviceManager& GetDeviceManager( void ) { return m_devMgr; }
	DeviceHandlerFactory& GetHandlerFactory( void ) { return m_deviceHandlerFactory; }
protected:
	// event handlers (these functions should _not_ be virtual)
	void OnAbout( wxCommandEvent& e );
	void OnConfigureLogOutput( wxCommandEvent& e );
	void OnQuit( wxCommandEvent& e );
	void OnSetID( wxCommandEvent& e );
	void OnTimer( wxTimerEvent& e );
	void OnUpdateDeviceList( wxCommandEvent& e );
	void OnUpdateFirmware( wxCommandEvent& e );
	void OnUpdateKernelDriver( wxCommandEvent& e );
	enum TTimerEvent
	{
		teListUpdate
	};
private:
	void								BuildList( void );
	void								UpdateDeviceList( void );
	mvIMPACT::acquire::DeviceManager	m_devMgr;
	DeviceListCtrl*						m_pDevListCtrl;
	wxMenuItem*							m_pMIActionSetID;
	wxMenuItem*							m_pMIActionUpdateFW;
	wxMenuItem*							m_pMIActionUpdateKernelDriver;
	wxMenuItem*							m_pMIActionUpdateDeviceList;
	wxTextCtrl*							m_pLogWindow;
	wxTimer								m_listUpdateTimer;
	unsigned int						m_lastDevMgrChangedCount;
	LogConfigurationVector				m_debugData;
	DeviceHandlerFactory				m_deviceHandlerFactory;
	// any class wishing to process wxWidgets events must use this macro
	DECLARE_EVENT_TABLE()
	//-----------------------------------------------------------------------------
	// IDs for the controls and the menu commands
	enum TMenuItem
	//-----------------------------------------------------------------------------
	{
		miQuit = 1,
		miAbout,
		miAction_SetID = 1000,
		miAction_UpdateFW,
		miAction_UpdateKernelDriver,
		miAction_ConfigureLogOutput,
		miAction_UpdateDeviceList,
		miCOMMON_LAST
	};
	//-----------------------------------------------------------------------------
	enum TDeviceIDLimits
	//-----------------------------------------------------------------------------
	{
		didlMin = 0,
		didlMax = 250
	};
#ifdef BUILD_WITH_DIRECT_SHOW_SUPPORT
	DirectShowDeviceManager				m_DSDevMgr;
	//-----------------------------------------------------------------------------
	enum TMenuItem_DirectShow
	//-----------------------------------------------------------------------------
	{
		miDirectShow_RegisterAllDevices = miCOMMON_LAST,
		miDirectShow_UnregisterAllDevices,
		miDirectShow_SetFriendlyName
	};
	wxMenuItem*							m_pMIDirectShow_RegisterAllDevices;
	wxMenuItem*							m_pMIDirectShow_UnregisterAllDevice;
	wxMenuItem*							m_pMIDirectShow_SetFriendlyName;
	void OnRegisterAllDevicesForDirectShow( wxCommandEvent& e )
	{
		m_DSDevMgr.registerAllDevices();
		BuildList();
	}
	void OnUnregisterAllDevicesForDirectShow( wxCommandEvent& e )
	{
		m_DSDevMgr.unregisterAllDevices();
		BuildList();
	}
	void OnSetFriendlyNameForDirectShow( wxCommandEvent& e );
public:
	int SetDSFriendlyName( int deviceIndex );
#endif // #ifdef BUILD_WITH_DIRECT_SHOW_SUPPORT
};

#endif // DeviceConfigureFrameH
